#include "stdafx.h"
#include "emule.h"
#include "LogListCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define DFLT_MAX_LOG_MESSAGES	1000

IMPLEMENT_DYNAMIC(CLogListCtrl, CListCtrl)

BEGIN_MESSAGE_MAP(CLogListCtrl, CListCtrl)
	ON_WM_CONTEXTMENU()
	ON_WM_SIZE()
END_MESSAGE_MAP()

CLogListCtrl::CLogListCtrl(){
	m_iMaxLogMessages = DFLT_MAX_LOG_MESSAGES;
	m_bAutoScroll = true;
}

CLogListCtrl::~CLogListCtrl(){
}

void CLogListCtrl::Localize(){
}

void CLogListCtrl::Init(){
	SetExtendedStyle(LVS_EX_FULLROWSELECT);

	InsertColumn(0, _T("Message"));

	m_LogMenu.CreatePopupMenu();
	m_LogMenu.AddMenuTitle(GetResString(IDS_LOGENTRY));
	m_LogMenu.AppendMenu(MF_STRING,MP_COPYSELECTED, GetResString(IDS_COPY));
	m_LogMenu.AppendMenu(MF_SEPARATOR);
	m_LogMenu.AppendMenu(MF_STRING,MP_SELECTALL, GetResString(IDS_SELECTALL));
	m_LogMenu.AppendMenu(MF_STRING,MP_REMOVEALL, GetResString(IDS_PW_RESET));
	m_LogMenu.AppendMenu(MF_SEPARATOR);
	m_LogMenu.AppendMenu(MF_STRING,MP_AUTOSCROLL, GetResString(IDS_AUTOSCROLL));

	m_iMaxLogMessages = theApp.glob_prefs->GetMaxLogMessages();
	if (m_iMaxLogMessages == 0)
		m_iMaxLogMessages = DFLT_MAX_LOG_MESSAGES;
}

void CLogListCtrl::AddEntry(LPCTSTR pszMsg){
	// 1) avoid flickering while inserting *and* deleting listview items
	// 2) avoid flickering while inserting listview items *and* calling 'EnsureVisible'
	SetRedraw(FALSE); 

	LVITEM Item;
	Item.mask = LVIF_TEXT;
	Item.iItem = INT_MAX;
	Item.iSubItem = 0;
	Item.pszText = (LPTSTR)pszMsg;
	int iItem = InsertItem(&Item);
	if (iItem != -1){
		if (m_iMaxLogMessages != -1){ // '-1' ... No limit!
			if (GetItemCount() > m_iMaxLogMessages)
				VERIFY( DeleteItem(0) );
			ASSERT( GetItemCount() <= m_iMaxLogMessages );
		}
		if (m_bAutoScroll)
			EnsureVisible(iItem, FALSE);
	}

	SetRedraw(TRUE);
}

void CLogListCtrl::Reset(){
	DeleteAllItems();
}

void CLogListCtrl::OnSize(UINT nType, int cx, int cy){
	CListCtrl::OnSize(nType, cx, cy);
	CRect rcClient;
	GetClientRect(&rcClient);
	SetColumnWidth(0, rcClient.Width());
}

void CLogListCtrl::OnContextMenu(CWnd* pWnd, CPoint point){
	int iItemCount = GetItemCount();
	UINT uSelCount = GetSelectedCount();
	m_LogMenu.EnableMenuItem(MP_COPYSELECTED, uSelCount > 0 ? MF_ENABLED : MF_GRAYED);
	m_LogMenu.EnableMenuItem(MP_REMOVEALL, iItemCount > 0 ? MF_ENABLED : MF_GRAYED);
	m_LogMenu.EnableMenuItem(MP_SELECTALL, iItemCount > 0 ? MF_ENABLED : MF_GRAYED);
	m_LogMenu.CheckMenuItem(MP_AUTOSCROLL, m_bAutoScroll ? MF_CHECKED : MF_UNCHECKED);

	if (point.x == -1 && point.y == -1){
		int iIdxItem = GetNextItem(-1, LVNI_SELECTED | LVNI_FOCUSED);
		if (iIdxItem != -1){
			RECT rc;
			if (GetItemRect(iIdxItem, &rc, LVIR_BOUNDS)){
				point.x = rc.left + GetColumnWidth(0) / 2;
				point.y = rc.top + (rc.bottom - rc.top) / 2;
				ClientToScreen(&point);
			}
		}
		else{
			point.x = 16;
			point.y = 32;
			ClientToScreen(&point);
		}
	}
	m_LogMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON, point.x, point.y, this);
}

BOOL CLogListCtrl::OnCommand(WPARAM wParam, LPARAM lParam){
	switch (wParam) {
	case MP_COPYSELECTED:{
		CString strItems;
		int iItem = -1;
		while ((iItem = GetNextItem(iItem, LVNI_SELECTED)) != -1){
			if (!strItems.IsEmpty())
				strItems += _T("\r\n");
			strItems += GetItemText(iItem, 0);
		}
		theApp.CopyTextToClipboard(strItems);
		break;
	}
	case MP_SELECTALL:
		SetItemState(-1, LVIS_SELECTED, LVIS_SELECTED);
		break;
	case MP_REMOVEALL:
		Reset();
		break;
	case MP_AUTOSCROLL:
		m_bAutoScroll = !m_bAutoScroll;
		break;
	}
	return TRUE;
}

CString CLogListCtrl::GetLastLogEntry(){
	CString strLog;
	int iItems = GetItemCount();
	if (iItems > 0)
		strLog = GetItemText(iItems - 1, 0);
	return strLog;
}

CString CLogListCtrl::GetAllLogEntries(){
	CString strLog;
	int iItemCount = GetItemCount();
	for (int i = 0; i < iItemCount; i++)
		strLog += GetItemText(i, 0) + _T("\r\n");
	return strLog;
}
